import { Blog, BlogAdminsTrait } from './types';
import { z, ZodType } from 'zod';
import { Subset } from '../utils/types';
import { userIdParser } from '../user/parsers';
import { isChannelId } from '../channel/utils';
import { channelIdParser } from '../channel/parsers';
import { nullableBoolParser, nullableStringParser, nullableUrlStringParser } from '../common/parsers';
import { paypalPlanIdParser, paypalProductIdParser } from '../paypal/parsers';
import { stripeProductIdParser } from '../stripe/parsers';
import { isBlogId } from './id.utils';
import { firestoreListParamsParser } from '../common/parsers/queryParams';
const paypalPlanIdOrNull = paypalPlanIdParser.or(z.null());
const paypalProductIdOrNull = paypalProductIdParser.or(z.null());
export const blogBeneficiaryTraitParser = z.object({
  admins: z.array(userIdParser),
});

export const blogListQueryParser = firestoreListParamsParser;
export const blogListResultParser = z.array(z.any(/*stub*/)).transform((a) => a as Blog[]);

export const blogIdParser = z.string().refine(isBlogId);

export const blogGetOneParamsParser = z.object({
  id: blogIdParser,
});
export const blogGetOneResultParser = z.any(/*stub, not schemable*/).transform((b) => b as Blog);
export const blogDeleteParamsParser = blogGetOneParamsParser;

export const blogAdminParser = z.string(); // TODO make sense of what we store here

export const makeBlogAdminsTraitParser = <T1, T extends ZodType<T1>>(a: T) =>
  z.object({
    admins: a,
  });
export const makeBlogAdminsParser = <T extends string, P extends z.ZodType<T>>(p: P) => z.array(p);

export const blogAdminsTraitParser = makeBlogAdminsTraitParser(makeBlogAdminsParser(blogAdminParser));

// eslint-disable-next-line
type AssertBlogBeneficiaryTraitParser1 = Subset<z.infer<typeof blogBeneficiaryTraitParser>, BlogAdminsTrait>;
// eslint-disable-next-line
type AssertBlogBeneficiaryTraitParser2 = Subset<BlogAdminsTrait, z.infer<typeof blogBeneficiaryTraitParser>>;
export const blogCreateInputParser = z
  .object({
    aboutArtist: z.string(),
    artist: z.string(),
    artistPicSmall: nullableUrlStringParser,
    artistPicURL: nullableUrlStringParser,
    //I think only one artist has beneficiaries, which is an object described as {NAME: image_link}. Not validating for now
    // beneficiaries:
    beneficiaryAccepted: nullableBoolParser,
    //collectiveSubscribers seem to all be an object with x amt of keys that each have a value of an empty list []. Will have to custom validate this,TODO
    // collectiveSubscribers:
    communityPage: nullableBoolParser,
    coverPicSmall: nullableUrlStringParser,
    coverPicURL: nullableUrlStringParser,
    coverPicURLSmall: nullableUrlStringParser,
    coverResolutions: z.object({
      small320: nullableUrlStringParser,
      small800: nullableUrlStringParser,
      small1280: nullableUrlStringParser,
      small1600: nullableUrlStringParser,
      small2000: nullableUrlStringParser,
    }),
    coverSmall: nullableUrlStringParser,
    curatorAlpha: nullableBoolParser,
    createdAllowReminder: nullableBoolParser,
    customlink: nullableUrlStringParser,
    description: nullableStringParser,
    emailNewSubscribers: nullableBoolParser,
    emailSettings: z.object({
      reminderTwo: nullableBoolParser,
      newSub: nullableBoolParser,
      reminderThree: nullableBoolParser,
      reminderFour: nullableBoolParser,
      emailNewSubscriber: nullableBoolParser,
      emailNewSubscriberCollective: nullableBoolParser,
    }),
    featuredArtists: z.array(z.string().refine((id) => isChannelId(id))).default([]),
    founder: z.union([z.boolean(), z.null()]).default(null),
    freetier: z.boolean().default(true),
    frequency: z.string().default('0'),
    launchDraft: nullableStringParser,
    launched: z.boolean().default(false),
    linkTitles: z.array(z.string()).default([]),
    //TODO: linkViews needs to be a computed value/not validated on creation
    // linkViews: z.array(z.number().default(0)),
    links: z.array(z.string().url()).default([]),
    paypal: nullableBoolParser,
    'paypal_plan_2.5': paypalPlanIdOrNull,
    paypal_plan_5: paypalPlanIdOrNull,
    paypal_plan_10: paypalPlanIdOrNull,
    paypal_product_id: paypalProductIdOrNull,
    posts: z.array(channelIdParser).default([]),
    price: z.union([z.number(), z.null()]).default(null),
    profileResolutions: z.object({
      small64: nullableUrlStringParser,
      small128: nullableUrlStringParser,
      small256: nullableUrlStringParser,
      small480: nullableUrlStringParser,
    }),
    reminderLaunchSent: nullableBoolParser,
    reverseTitle: nullableBoolParser,
    scheduled: nullableBoolParser,
    show: nullableStringParser,
    //TODO: ensure we are never overiding with custom ids. shouldnt be the case. Also, worth verifying this with a function that checks with stripe... maybe
    stripeProductID: stripeProductIdParser,
    subscribers: z.array(channelIdParser).default([]),
    tipButton: nullableBoolParser,
    title: z.string().default('currents'),
    verified: z.boolean().default(false),
  })
  .merge(blogBeneficiaryTraitParser);
export const blogCreateOutputParser = blogIdParser;
export type CreateBlogRequest = z.infer<typeof blogCreateInputParser>;
export type CreateBlogResponse = z.infer<typeof blogCreateOutputParser>;

export const blogUpdateInputParser = blogCreateInputParser.partial();
export const blogUpdateParamsParser = blogGetOneParamsParser;
export const blogUpdateResultParser = z.void();
